# Datei: regelung.py (Python 3)
# Autor: gkemnitz
# letzte Aenderung: 28.11.2016 
#
# Steuerung einer verketteten Bewegungsfolge.
# Speicherung die Winkelabweichungen, Sollwerte und Integralanteile
# tabellarische und graphische Darstellung.
#
# Auf dem Mikrorechner muss das Programm "test_regelung" laufen.
# Als COM-Schnittstelle die der seriellen Verbindung einstellen.


import serial                   # serielle Schnittstelle
import matplotlib.pyplot as plt # Plotfunktion 
from sys import exit            # Programmabbruch 

kp = 1000; ki=500;              # Regelungskoeffizienten
ts = 5                          # Regelschritte je Nachricht

# Bewegungsablauftupen (speed, count)
#  speed: Geschwindigkeit in Winkelschritten*256 je 20ms
#  count Anzahl von Schritten der Dauer ts*20ms
trajList = [(2000, 20), (1000, 20), (-3000, 20)]

def smsg(v):                    # Sendenachricht erzeugen
    bv = v.to_bytes(2, byteorder='big', signed=True)
    bts = ts.to_bytes(2, byteorder='big')
    bkp = kp.to_bytes(2, byteorder='big')
    bki = ki.to_bytes(2, byteorder='big')
    return bv + bv + bts + bkp + bki

# Oeffnen der seriellen Schnittstelle mit Timeout
# Schnittstellenbezeichnung, hier COM3, anpassen
ser = serial.Serial("COM9", timeout=ts*4)    

t =[0]; dR=[0]; dL=[0]; s=[0]    # Leerlisten Zeit, ..
print(' t |speed|diff_R|intg_L|diff_L|intg_B|')

ser.write(smsg(trajList[0][0]))  # eine Schreibop. auf Vorrat
for (speed, count) in trajList:
    for idx in range(count):     # fuer count Messschritt zu 0,1s
        ser.write(smsg(speed))
        rmsg = ser.read(8)    
        if len(rmsg)<8:          # wenn kein Datenempfang 
           ser.close();          # Schnittstelle schliessen
           print('Datenempfang: %s (keine 8-Byte-Nachricht, Abbruch)'%rmsg)
           exit()                # Script beenden
        diff_R = int.from_bytes(rmsg[0:2], byteorder='big', signed=True)
        intg_L = int.from_bytes(rmsg[2:4], byteorder='big', signed=True)
        diff_L = int.from_bytes(rmsg[4:6], byteorder='big', signed=True)
        intg_B = int.from_bytes(rmsg[6:8], byteorder='big', signed=True)
        print('%3.1f|'%t[-1] + '%5i|'%speed, '%5i|'%diff_R, '%5i|'%intg_L, '%5i|'%diff_L, '%5i|'%intg_B)
        t  += [t[-1]+0.1]
        dR += [diff_R]
        dL += [diff_L]
        s  += [speed*(10.0/256)]
    
ser.close()                      # serielle Verbindung schliessen
plt.plot(t, dR, 'r', t, dL, 'b', t, s, 'k')
plt.xlabel('Zeit in s')          # Achsen beschriften
plt.ylabel('Fehler in Winkelschritten (ws)\nSollgeschwindigkeit in ws/200ms')
plt.show()   
 
